package jrobocom;

import java.io.*;
import java.net.*;
import java.util.*;

/***************************************************************
****************************************************************/
public class DelegateEventThread extends Thread
{
	Socket socket;
	ObjectInputStream  oInStr;
	ObjectOutputStream oOutStr;
	RoboAdapter adapter;
	String appName;
	boolean appOK;

	/*************************************************************************
  *************************************************************************/
	public DelegateEventThread(String name, Socket s, RoboAdapter adapter)
	{
		super(name);
		this.socket = s;
		this.adapter = adapter;
		this.appOK = true;

		try{
			oOutStr=new ObjectOutputStream(s.getOutputStream()); //for writing actions from matrix to java-device
			oInStr=new ObjectInputStream(s.getInputStream());   //for reading events from java-device to matrix
			System.out.println(this.getName()+": O LegoAdapter connected");
		}
		catch(IOException e){
			System.out.println(this.getName()+": X Problem while LegoAdapter connected: "+e);
		}
	}

	/*************************************************************************
  *************************************************************************/
	public void run()
	{
		boolean running = true;

		try{
			while(running){
			try{
				System.out.println("waiting for adapterobject");
				String cmd = (String)oInStr.readObject();
				System.out.println(this.getName()+": READ FROM APPLET: <"+cmd+">");

				if (cmd != null){
					StringTokenizer tok = new StringTokenizer(cmd);
					String cmdHost = null;

					cmdHost = getHostName();
/*
					while (tok.hasMoreTokens())
						cmdHost = tok.nextToken();

					//localhost problems
					if (cmdHost.equals("127.0.0.1"))
						cmdHost = "localhost.localdomain";
*/
					if (cmd.startsWith("PING")){
						System.out.println(this.getName()+"got PING");
						this.appOK = true;
					}
					else if (cmd.startsWith("JROBO INIT")){
						if (adapter.freeControl()) {
							System.out.println("@@@@@ Control FREE -> get control");
							adapter.sendToAdapterX(cmdHost, this.getName(), "JROBO FREE");
						}
						else{
							System.out.println("@@@@@ Control NOT free -> wait");
							adapter.sendToAdapterX(cmdHost, this.getName(), "JROBO NOTFREE");
						}
					}
					else if (cmd.startsWith("JROBO CON")){
						if (adapter.getControl(this)){
							adapter.sendToAdapterX(cmdHost, this.getName(), "JROBO ONLINE");
							adapter.sendToAdapterNotX(cmdHost, this.getName(), "JROBO NOTFREE");
						}
						else adapter.sendToAdapterX(cmdHost, this.getName(), "JROBO OFFLINE");
					}
					else if (cmd.startsWith("JROBO DISCON")){
						adapter.forgetControl();
						adapter.sendToAdapterX(cmdHost, this.getName(), "JROBO OFFLINE");
						adapter.sendToAdapterNotX(cmdHost, this.getName(), "JROBO FREE");
					}
					else adapter.sendToRobot(cmd);
				}
			}
			catch(IOException e){
				System.out.println(this.getName()+": Couldn't get I/O for connection: "+e);
				running = false;
				adapter.stopMe(this);
			}
			}
		}

		catch(ClassNotFoundException e)
		{
			System.out.println(this.getName()+": "+e);
			//System.exit(1);		
		}
		finally{
			try{ socket.close(); }
			catch(IOException e){}
		}
		//this.destroy();
	}

  /*****************************************************************
	******************************************************************/
  public void destroy()
	{
		System.out.println(this.getName()+": destroyed!");
	
		try{
			socket.close();
			socket = null;
			oInStr.close();
			oInStr = null;
  		oOutStr.close();
			oOutStr = null;
		}
		catch(IOException e){
			System.out.println(this.getName()+": "+e);
		}
  }
	
	/*************************************************************************
  *************************************************************************/	
	public String getHostName()
	{
		return(this.socket.getInetAddress().getHostName());
	}

	/*************************************************************************
  *************************************************************************/	
	public void setAppName(String applet)
	{
		this.appName = applet;
	}

	/*************************************************************************
  *************************************************************************/	
	public String getAppName()
	{
		return(this.appName);
	}

	/*************************************************************************
  *************************************************************************/	
	public boolean isOK()
	{
		return(this.appOK);
	}

	/*************************************************************************
  *************************************************************************/	
	public void setNotOK()
	{
		this.appOK = false;
	}
}
